package org.computer.aman.metrics.size.loc;

import java.io.FileReader;
import java.io.IOException;
import java.io.LineNumberReader;
import java.util.Iterator;

import org.computer.aman.io.sourcecode.NotSupportedSourceFileExeption;
import org.computer.aman.io.sourcecode.SourceCodeLine;
import org.computer.aman.io.sourcecode.SourceFile;
import org.computer.aman.metrics.util.CodeLineMap;
import org.computer.aman.metrics.util.CodeMap;

/**
 * ñΥեˤĤ LOC ¬Ȥ˴Ϣ뵡ǽ
 * <p>
 * @author Hirohisa AMAN &lt;aman@computer.org&gt;
 * @version $Id: LOC.java,v 1.5 2009/03/27 07:33:10 aman Exp $ 
 */
public class LOC
{
    /**
     * LOC ¬¹Ԥ̤ ResultSet 󥹥󥹤Ȥ֤
     * 
     * @param aSourceFile ¬оݤΥե
     * @return LOC ¬η
     * @throws IOException 
     * @throws NotSupportedSourceFileExeption ݡȳΥեξ
     */
    public static LOCResultSet measure(final SourceFile aSourceFile)
    throws NotSupportedSourceFileExeption, IOException
	{
        return measure(aSourceFile, 1, -1);
	}
    
    /**
     * LOC ¬¹Ԥ̤ ResultSet 󥹥󥹤Ȥ֤
     * 
     * @param aSourceFile ¬оݤΥե
     * @param aBeginLineNumber ¬γϹ
     * @return LOC ¬η
     * @throws IOException 
     * @throws NotSupportedSourceFileExeption ݡȳΥեξ
     */
    public static LOCResultSet measure(final SourceFile aSourceFile, final int aBeginLineNumber)
    throws NotSupportedSourceFileExeption, IOException
    {
        return measure(aSourceFile,aBeginLineNumber,-1);
    }

    /**
     * LOC ¬¹Ԥ̤ ResultSet 󥹥󥹤Ȥ֤
     * 
     * @param aSourceFile ¬оݤΥե
     * @param aBeginLineNumber ¬γϹ
     * @param anEndLineNumber ¬νλԡ-1 ξޤǡ
     * @return LOC ¬η
     * @throws IOException 
     * @throws NotSupportedSourceFileExeption ݡȳΥեξ
     */
    public static LOCResultSet measure(final SourceFile aSourceFile, final int aBeginLineNumber, final int anEndLineNumber)
    throws NotSupportedSourceFileExeption, IOException
    {
        // եγĥҤ¬ǽɤȽ
        final String EXT = aSourceFile.getExtension();
        if ( !EXT.equalsIgnoreCase("java") &&
             !EXT.equalsIgnoreCase("c") && !EXT.equalsIgnoreCase("h") && !EXT.equalsIgnoreCase("cpp") &&
             !EXT.equalsIgnoreCase("cc") && !EXT.equalsIgnoreCase("cxx") && !EXT.equalsIgnoreCase("m") &&
             !EXT.equalsIgnoreCase("i") && !EXT.equalsIgnoreCase("ii") && !EXT.equalsIgnoreCase("h") ){
            throw new NotSupportedSourceFileExeption("not supported file type: " + aSourceFile);
        }
        
        // ¬̤뤿Υ֥
        LOCResultSet result = new LOCResultSet(aSourceFile);
        
        // ¬оݤΥɤˤĤơɥޥåפ
        CodeMap map = CodeMap.create(aSourceFile);
        
        // ¬̥֥ȤإɤƤिᡤ reader Ѱդ
        LineNumberReader reader = new LineNumberReader(new FileReader(aSourceFile));
        
        // 夫Ԥĥޥåפ򸫤Ƥ
        Iterator<CodeLineMap> itr = map.iterator();
        int loc = 0;
        int lineNumber = 0;        
        while ( itr.hasNext() ){
            CodeLineMap lineMap = itr.next();
            lineNumber++;
            if ( lineNumber < aBeginLineNumber ){
                continue;
            }
            if ( anEndLineNumber > 0 && anEndLineNumber < aBeginLineNumber ){
                continue;
            }
            if ( lineNumber-1 == anEndLineNumber ){
                break;
            }

            // ܹԤƤ SourceCodeLine ֥ȤȤ
            SourceCodeLine codeLine = new SourceCodeLine(reader.readLine());

            // LOC ͤ򥫥Ȥʤ顤¸ɤιԤΤߤ˹ֹդƤ
            if ( lineMap.getCodeCount() > 0 ){
                loc++;
                codeLine.setLineNumber(loc);
                result.incrementLOC();
            }
            else{                
                codeLine.setLineNumber(-1);
                if ( lineMap.isBlankLine() ){
                    result.incrementBlankCount();
                }
            }
            result.addContents(codeLine);
        }

        reader.close();
        
        return result;        
    }
}
